//*****************************************************************************************
// Counter-Strike 1.6 Bot
// Filename: Bot_Engine.h
// Description: Counter-Strike 1.6 Bot Main Engine Header
// Author: He@dShot
//*****************************************************************************************
// Classes & Structures
//*****************************************************************************************
#ifndef BOT_ENGINE_INCLUDED
#define BOT_ENGINE_INCLUDED
//*****************************************************************************************
// BOT CLASS
//*****************************************************************************************
typedef struct
{
   // BOOLEAN
   bool b_CanUseRadioChat;    // TRUE if bot is able to use radio or voice commands
   bool b_UsesVoiceChat;	  // TRUE if bot uses bot chat files instead of radio
   bool b_UsesRadioChat;      // TRUE if bot uses radio instead of bot chat files 
   bool b_IsLeader;           // TRUE if bot is leader of his team
   bool b_IsAlive;            // TRUE if bot is alive
   bool b_IsVIP;              // TRUE if bot is VIP
   bool b_HasC4Bomb;          // TRUE if bot has the bomb
   bool b_HasDefuserKit;      // TRUE if bot has defuser kit
   bool b_IsPlantingBomb;     // TRUE if bot is planting the bomb
   bool b_IsDefusingBomb;     // TRUE if bot is defusing the bomb
   bool b_IsInBombZone;       // TRUE if bot is in the bomb zone
   bool b_HasFoundHostages;   // TRUE if bot has found the hostages location
   bool b_CanReload;		  // TRUE if bot can reload his weapon
   bool b_IsReloading;        // TRUE if bot is reloading his weapon
   bool b_CanThrowGrenade;    // TRUE is bot can throw a grenade. Check "g_GrenadeThrowConditions" for more info...
   bool b_IsBlinded;          // TRUE if bot has been hit by a flashbang grenade
   bool b_LeaderChosen;       // TRUE if the bots have chosen a leader from the team
   bool b_HasChosenTeam;      // TRUE if the bot has chosen a team ingame
   bool b_HasChosenModel;     // TRUE if the bot has chosen a model appearance
   bool b_HasEnteredGame;     // TRUE if the bot has entered the game
   bool b_HasSpawned;         // TRUE if the bot has spawned in the game
   bool b_SlotUsed;           // TRUE if the bot's player slot is used

   // CHAR
   char c_Skill_Suffix[9];              // Bot skill suffix

   // FLOAT
   float f_CurrentMoveSpeed;			// Current speed forward/backward
   float f_CurrentSideMoveSpeed;		// Current speed when strafing left/right
   float f_MinSpeedNormal;				// Minimum speed when running (not walking)
   float f_MaxSpeedNormal;		        // Maximum speed when running (not walking)
   float f_BlindRecoverDelay;			// Time in seconds to recover sight when blinded by flashbang
   float f_MinBlindRecoverDelay;        // Minimum time in seconds to recover sight when blinded by flashbang   min: 5
   float f_MaxBlindRecoverDelay;        // Maximum time in seconds to recover sight when blinded by flashbang
   float f_NextRadioCommandDelay;		// Time between radio commands & responses
   float f_NextVoiceCommandDelay;		// Time between voice commands & responses
   float f_ReloadCheckDelay;			// Time between next weapon reload check
   float f_ItemPickupDelay;             // Time between searching/picking up an item on the ground
   float f_KnifeSwitchTime;             // Time to walk with knife at the beggining of the round
   float f_ZoomInOutDelay;              // Time between zooming in and out with sniper weapons            

   // INT
   int i_CurrentWeapon;                 // Bot current Weapon
   int i_CurrentPersonality;            // Bot current Personality
   int i_PrefferedWeapon;               // Bot Preferred Weapon
   float f_CurrentVoicePitch;           // Bot Voice Pitch
   int i_CurrentSkill;                  // Bot skill
   int i_CurrentFrags;                  // Bot frags
   int i_currentNetMessage;             // Bot current network message

   // Game stuff
   int i_SelectedTeam;                  // Bot's selected team when creating bot 
   int i_SelectedModel;                 // Bot's selected model when creating bot
   const char *i_SelectedName;          // Bot's selected name when creating bot
   
   //edict_t & vector
   edict_t *pEdict;                     // link to bot entity
   edict_t *enemy;                      // link to enemy player entity
   edict_t *hostage[8];      // link to hostage entity
   edict_t *m_friend;                   // link to team-mate player entity
   edict_t *vip_target;                 // link to VIP player entity
   edict_t *droppedBomb;                // link to dropped bomb entity
   edict_t *pickup_item;				// link to pickup item entity
   Vector v_origin;
   Vector v_eyeorigin;
   Vector v_aiming_location;
   Vector v_angle;
   Vector v_ideal_angles;
   Vector v_randomization;
   Vector v_randomized_ideal_angles;
   Vector v_angular_deviation;
   Vector v_aim_speed;
   Vector v_target_angular_speed;
   float f_randomize_angles_delay;
   float f_player_target_delay;
   float f_last_shoot_delay;

   //------------------------------------------------------------------------------
   float f_minReactionDelay;        // Bot minimum firing delay when spoted an enemy
   float f_maxReactionDelay;        // Bot maximum firing delay when spoted an enemy
   //----------------------------------------------------
   float f_campMinimumStartDelay;   // Bot minimum delay before starting to camp
   float f_campMinimumEndDelay;     // Bot minimum delay before ending to camp
   //----------------------------------------------------
   float f_minTurnSpeed;            // Bot minimum turning speed
   float f_maxTurnSpeed;            // Bot maximum turning speed
   //----------------------------------------------------
   int i_HeadshotPercent;           // Bot probability of aiming at head
   //----------------------------------------------------
   int i_HearShootPercent;          // Bot probability of shooting through wall/door when hearing something
   int i_SeenShootPercent;          // Bot probability of shooting through wall/door when seeing something 
   int i_RecoilPauseAmount;         // Bot weapon recoil amount needed to pause firing weapon
   //----------------------------------------------------
   int i_campingPercent;            // Bot percent of camping troughout the game
   int i_bombGuardingPercent;       // Bot percent of droping the dropped bomb troughout the game
   int i_bombSiteGuardingPercent;   // Bot percent of guarding bombsites troughout the game
   int i_bombApproachWalkPercent;   // Bot percent of walking when spoting the dropped bomb
   int i_cornerWalkingPercent;      // Bot percent of walking when approaching corners

   //------------------------------------------------------------------------------
} Bot_t;
//****************************************************************************************
// Function Prototypes
//****************************************************************************************
   // Bot
   edict_t GetNewEnemy(Bot_t *bot, edict_t *oldEnemy); // New enemy finding function
   void BotAI(Bot_t *bot);							// The AI function
   void PurchaseWeapons(Bot_t *bot);           // Weapon buying function
   void InstantRadioMessage(Bot_t *bot,int Type);   // Radio Message function
   void InstantVoiceMessage(Bot_t *bot,int Type);   // Voice Message function
   void ThrowGrenadeAtLocation(Bot_t *bot, Vector location, int GrenadeType);	 // Grenade throwing function Types: -HeGrenade, -SmokeGrenade, -Flashbang
   void FireWeapon(Bot_t *bot);						 // Weapon Firing function
   void RemoveChatterIcon(Bot_t *bot);         // Chatter Icon Removal function
   // bool functions
   bool HasGrenades(Bot_t *bot);               // TRUE if bot has any grenade
   bool HasPrimaryWeapon(Bot_t *bot);			 // TRUE if bot has a PRIMARY weapon
   bool HasSecondaryWeapon(Bot_t *bot);		 // TRUE if bot has a SECONDARY weapon
   bool UsesWeakWeapon(Bot_t *bot);			 // TRUE if bot uses a weak weapon like the scout or tmp
   bool HasEnemyInSight(Bot_t *bot, edict_t *enemy);			 // TRUE if bot has an enemy in his viewcone
   bool IsFriendInLineOfFire(Bot_t *bot, edict_t *teammate);  // TRUE if a bot's team-mate is in his aim
   bool IsEnemyProtectedByShield(Bot_t *bot, edict_t *enemy);  // TRUE if the enemy is protected by a shield
   bool IsBombOutOfTime(float Time);     // TRUE if bomb is out of time
   bool IsBehindSmokeClouds(edict_t *ent); // TRUE if the entity is behind smoke clouds
   bool	IsEntityVisibleToEntity(edict_t *entity, edict_t *target); // TRUE if the entity2 is visible to target entity
   bool UsesPistol(Bot_t *bot);                // TRUE if the bot is using a pistol
   bool UsesRifle(Bot_t *bot);
   bool UsesSniper(Bot_t *bot);
   bool UsesGrenade(Bot_t *bot);
   bool UsesMachineGun(Bot_t *bot);
   bool UsesSubMachineGun(Bot_t *bot);
   bool UsesShotgun(Bot_t *bot);
   bool HasPistol(Bot_t *bot);
   bool HasRifle(Bot_t *bot);
   bool HasSniper(Bot_t *bot);
   bool HasGrenades(Bot_t *bot, int grenadeType);
   bool HasMachineGun(Bot_t *bot);
   bool HasSubMachineGun(Bot_t *bot);
   bool HasShotgun(Bot_t *bot);

   //Misc
   extern int pfnRegUserMsg(const char *pszName, int iSize);
   extern int GetMsecValue (void);
   extern void FakeClientCommand(edict_t *pBot, char *arg1, char *arg2, char *arg3);

   // Bot Manager
   void CreateBot(const char *Name, int Team, const char *Model, int Personality, int Skill); // Bot creation function
   void CreateRandomBot(void);   // Random Bot creation function
   void KillBot(Bot_t *bot);   // Bot killing function
   void KillAllBots(void);       // Complete bot killing function
   void RemoveBot(Bot_t *bot); // Bot removal function
   void RemoveAllBots(void);     // Complete bot removal function
   void SetPingTag(Bot_t *bot, char pingTag);// Bot ping tag function

   int GetMaxClients (void);
   float GetWorldTime (void);
   void StartFrame(void);      // StartFrame()

   // Functions that handle client messages
   extern void BotMessage_CS_VGUI (void *p, int bot_index);
   extern void BotMessage_CS_ShowMenu (void *p, int bot_index);
   extern void BotMessage_CS_StatusIcon (void *p, int bot_index);
   extern void BotMessage_CS_WeaponList (void *p, int bot_index);
   extern void BotMessage_CS_CurrentWeapon (void *p, int bot_index);
   extern void BotMessage_CS_AmmoX (void *p, int bot_index);
   extern void BotMessage_CS_AmmoPickup (void *p, int bot_index);
   extern void BotMessage_CS_Damage (void *p, int bot_index);
   extern void BotMessage_CS_Money (void *p, int bot_index);
   extern void BotMessage_CS_DeathMsg (void *p, int bot_index);
   extern void BotMessage_CS_ScreenFade (void *p, int bot_index);
   extern void BotMessage_CS_SayText (void *p, int bot_index);
   extern void BotMessage_CS_HLTV (void *p, int bot_index);
   extern void BotMessage_CS_BombDrop (void *p, int bot_index);
   extern void BotMessage_CS_BombPickup (void *p, int bot_index);
   extern void BotMessage_CS_TextMsgAll (void *p, int bot_index);
   extern void BotMessage_CS_BotVoice (void *p, int bot_index);
   extern void BotMessage_CS_SendAudio (void *p, int bot_index);
   extern void BotMessage_CS_BarTime (void *p, int bot_index);
   extern void BotMessage_CS_TextMsg (void *p, int bot_index);

   void (*BotMessageFunction) (void *, int) = NULL;
#endif